@@ -66,12 +66,12 @@
 
 
 #define ITK_MAJOR_VERSION	4
-#define ITK_MINOR_VERSION	0
-#define ITK_RELEASE_LEVEL	TCL_RELEASE
-#define ITK_RELEASE_SERIAL	2
+#define ITK_MINOR_VERSION	1
+#define ITK_RELEASE_LEVEL	TCL_FINAL_RELEASE
+#define ITK_RELEASE_SERIAL	0
 
-#define ITK_VERSION		"4.0"
-#define ITK_PATCH_LEVEL		"4.0.2"
+#define ITK_VERSION		"4.1"
+#define ITK_PATCH_LEVEL		"4.1.0"
 
 
 /*
